/***************************************************************************************************
*                                           SENTINEL PROGRAM
****************************************************************************************************
* NAME:  I9I10map.sas
*
* PURPOSE: Import GEMS mappings to convert to SAS, generate distinct I10 lists, and output descriptives
*
* MAJOR STEPS: 1) Import GEMS and retain relevant records only
*              2) List distinct I10 codes per mapping
*              3) Output descriptive figures per each mapping
*
* KEY DEPENDENCIES/CONSTRAINTS/CAVEATS:
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@sentinelsystem.org
*
*--------------------------------------------------------------------------------------------------
* HISTORY:
*  Create date (mm/dd/yy):
*  Last modified date (mm/dd/yy): 
*  Version: v1.0
*
*  CHANGE LOG: 
*
*   Version   Date       Comment (reference external documentation when available)
*   -------   --------   -------------------------------------------------------------------------
*   1.0.0     05/30/18   First draft version
*
**************************************************************************************************/

/* System options */
options nosymbolgen nomlogic ;
options ls=100 nocenter ;

*** ------------------------------------------------------------------------------- ***;
*** USER PARAMETERS                                                                 ***;
*** ------------------------------------------------------------------------------- ***;

    *** INPUT PARAMETERS                                                            ***;                
    
    %let GEMS_FY = 2018; /* fiscal year of GEMS files retrieved by user to be processed */

    %let indsn_subsetI9 = ; /* input xls file that lists I9 codes to include in analysis. User must specified file extension in parameter value */
    %let invar_subsetI9 = ID9TEST; /* name of variable in input xls file that represents I9 codes to be translated */

    %let indsn_gemsI9  = 2018_I9gem.txt; /* input GEMS I9 file. User must specified file extension in parameter value */
    %let indsn_gemsI10 = 2018_I10gem.txt; /* input GEMS I10 file. User must specified file extension in parameter value */



    *** OUTPUT PARAMETERS                                                            ***;       

    %let outdsn_gemsF  = gemsF_&GEMS_FY.; /* output forward GEMS SAS filename */
    %let outdsn_gemsB  = gemsB_&GEMS_FY.; /* output backward GEMS SAS filename */
    %let outdsn_gemsFB = gemsFB_&GEMS_FY.; /* output combined GEMS SAS filename */

    %let outdsn_ID10F  = gemsID10F_&GEMS_FY.;  /* output I10 list SAS filename, forward mapping list SAS filename */
    %let outdsn_ID10B  = gemsID10B_&GEMS_FY.;  /* output I10 list SAS filename, backward mapping list SAS filename */
    %let outdsn_ID10FB  = gemsID10FB_&GEMS_FY.; /* output I10 list SAS filename, combined forward and backward  */

    %let outdsn_descstats = gemsStats_&GEMS_FY.; /* output descriptive statistic SAS filename */



    *** LOCATION PARAMETERS                                                         ***;       

    %let inloc_i9xls =  ; /* location of input xls file that lists I9 codes */ 
    %let inloc_gems = C:\dev\test\icd_mapping\inputfiles\2018-ICD-10-CM-General-Equivalence-Mappings\; /* location of input GEMS forward and backward mappings files */

    %let outloc_map = C:\dev\test\icd_mapping\output; /* location to save final process output to */


    /* assign libname for output directory */
    libname out "&outloc_map.";


*** ------------------------ START PROGRAM ---------------------------------------- ***;



*** Macro to create I9 parameter                                                    ***;
%macro create_i9_inclusion_parm; 

    %* ---------------------------------------------------------------------------- ***;
    %* Import ID9 inclusion/subset list and load values into parameter to be used   ***;
    %* to execute subsequent logic                                                  ***;
    %* ---------------------------------------------------------------------------- ***;

    %global i9_include I9_IncludeCount;

	%* init to missing ;
	%let i9_include= ;
	%let I9_IncludeCount= .;


	%* if user parameters defined, import xls;
	%if      %length(&indsn_subseti9.) ne 0
	     and %length(&inloc_i9xls.)    ne 0
	%then %do;

            PROC IMPORT OUT= _ID9_List1 
	                    DATAFILE= "&inloc_i9xls.&indsn_subsetI9."
                        REPLACE; 
	        RUN;

			%* if valid dataset created from import, load I9s into macro parm ;
			%if %sysfunc(exist(_ID9_List1))=1
			%then %do;

                PROC SQL NOPRINT;

				    SELECT count(distinct(&invar_subsetI9.))
					   into :I9_IncludeCount
					   from _ID9_List1;


	                SELECT distinct quote(strip(compress(&invar_subsetI9.,,'p')))
	                    into :I9_Include
                        separated by ','
	                    FROM _ID9_List1;
                QUIT;

			%end;
	%end; 

	%put ;
	%put;
	%put -----------------------------------------------------------------------------------------;
	%put I9_Include Code Count = &I9_IncludeCount.;
	%put I9 Include Code List = &I9_Include.;
	%put -----------------------------------------------------------------------------------------;
	%put;
	%put;

%mend create_i9_inclusion_parm; 


*** Macro to create GEMS mapping files                                              ***;
%macro import_GEMS_output_maps;

    %* ---------------------------------------------------------------------------- ***;
    %* Import GEMS text files, convert to SAS datasets to create output map files   ***;
    %* ---------------------------------------------------------------------------- ***;

	%* forward translation file/ICD-9 to 10 mapping;
    DATA out.&outdsn_gemsF.;

	    retain ID9 ID10 FLAG;
        INFILE "&inloc_gems.&indsn_gemsI9.";
        INPUT ID9  :$5. 
              ID10 :$7. 
              FLAG :$15.;

	    %if %length(&I9_Include.) ne 0
		    %then if ID9 in (&I9_Include.);;
		

    run; 

	proc sort data = out.&outdsn_gemsF. NODUPKEY;
	    by _all_;
	run;

	%* backward translation file/ICD-10 to 9 mapping;
    data out.&outdsn_gemsB.;

	    retain ID9 ID10 FLAG;
        INFILE "&inloc_gems.&indsn_gemsI10.";

        input ID10 :$7. 
              ID9  :$5.  
              FLAG :$15.;

	    %if %length(&I9_Include.) ne 0
		    %then if ID9 in (&I9_Include.);;

    run;

	proc sort data = out.&outdsn_gemsB. NODUPKEY;
	    by _all_;
	run;
 

	%* set forward and backward files to create combined output;
	data out.&outdsn_gemsFB.;
	    set out.&outdsn_gemsF.
		    out.&outdsn_gemsB.;
	run;

	proc sort data = out.&outdsn_gemsFB. NODUPKEY;
	    by _all_;
	run;
 

%mend import_GEMS_output_maps;

*** Macro to output I10 lists and desc extracts, one per GEMS mapping               ***;
%macro output_I10_desc (in_dsn = , map_direction = , out_dsn = , out_desc = );

    %* ---------------------------------------------------------------------------- ***;
    %* Process mappings to output distinct ID10 & create descriptive stats file     ***;
    %* ---------------------------------------------------------------------------- ***;

		proc sort data = &in_dsn.
		          out  = &out_dsn. (keep = ID10) NODUPKEY;
		    by id10;
		run;

		proc sql;
		    create table &out_desc. as
			    select   "GEMS FY &GEMS_FY., &map_direction. map" as GEMS_Type length = 30
			           , count(*)                                 as MapCount
			           , count(distinct ID9)                      as DistID9Count
			           , count(distinct ID10)                     as DistID10Count

	                   %if %length(&I9_Include.) = 0
		               %then %do;
					       , .                                    as NoMatchID9Count
						%end;

	                   %if %length(&I9_Include.) ne 0
		               %then %do;
					       , &I9_IncludeCount.- CALCULATED  DistID9Count as NoMatchID9Count
						%end;
 

				from &in_dsn.;
		quit;

%mend output_I10_desc;
	 

*** execute macro calls                                                             ***;
%create_i9_inclusion_parm;

%import_GEMS_output_maps;

%output_I10_desc (  in_dsn         = out.&outdsn_gemsF.
	                 , map_direction  = forward
                     , out_dsn        = out.&outdsn_ID10F.
                     , out_desc       = _desc_&outdsn_gemsF. );
	 
%output_I10_desc (  in_dsn         = out.&outdsn_gemsB.
	                 , map_direction  = backward
                     , out_dsn        = out.&outdsn_ID10B.
                     , out_desc       = _desc_&outdsn_gemsB. );
	 
%output_I10_desc (  in_dsn         = out.&outdsn_gemsFB.
	                 , map_direction  = combined
                     , out_dsn        = out.&outdsn_ID10FB.
                     , out_desc       = _desc_&outdsn_gemsFB. );

*** Combine descriptive extracts to create one output file with descriptives per    ***;
*** each mapping type                                                               ***;

	data out.&outdsn_descstats.;

	   retain GEMS_Type MapCount NoMatchID9Count DistID9Count DistID10Count;

	   set _desc_&outdsn_gemsF. 
           _desc_&outdsn_gemsB.
           _desc_&outdsn_gemsFB. ;
	run;
 
*** ------------------------ END PROGRAM ------------------------------------------ ***;
